#!/bin/sh

# Tilefish - um organizador de janelas lado a lado para o desktop
# Copyright (C) 2014 Adriano Rafael Gomes <adrianorg@arg.eti.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
# INICIO DA CONFIGURACAO
#

#
# quantidade de pixels separando as janelas,
# incluindo as bordas decorativas e o espaco separador
#
OFFSET_X=6 #2 borda + 4 separador
OFFSET_Y=37 #33 borda + 4 separador

#
# FIM DA CONFIGURACAO
#

get_desktop_atual()
{
    wmctrl -d | grep '* DG' | tr -s ' '
}

get_desktop_atual_id()
{
    get_desktop_atual | cut -d ' ' -f 1
}

get_desktop_atual_wa()
{
    get_desktop_atual | cut -d ' ' -f 8,9
}

get_janelas_do_desktop_atual()
{
    wmctrl -G -l | egrep "^\w+\s+$(get_desktop_atual_id)\s" | tr -s ' '
}

get_dimensoes_do_desktop_atual()
{
    working_area=$(get_desktop_atual_wa)
    top_left=$(echo $working_area | cut -d ' ' -f 1)
    width_height=$(echo $working_area | cut -d ' ' -f 2)
    top=$(echo $top_left | cut -d ',' -f 2)
    left=$(echo $top_left | cut -d ',' -f 1)
    width=$(echo $width_height | cut -d 'x' -f 1)
    height=$(echo $width_height | cut -d 'x' -f 2)
    echo "$top $left $width $height"
}

tile()
{
    linhas="$1"
    colunas="$2"
    get_dimensoes_do_desktop_atual | while read top left width height
    do
        n=$(get_janelas_do_desktop_atual | wc -l)
        dw=$(echo "$width / $colunas" | bc)
        dw1=$(echo "$dw - $OFFSET_X" | bc)
        dh=$(echo "$height / $linhas" | bc)
        dh1=$(echo "$dh - $OFFSET_Y" | bc)
        lin=0
        col=0
        get_janelas_do_desktop_atual | cut -d ' ' -f 1-6 | while read id desktop l t w h
        do
            x=$(echo "($col * $dw) + $left" | bc)
            y=$(echo "($lin * $dh) + $top" | bc)
            wmctrl -i -r $id -e 0,$x,$y,$dw1,$dh1
            col=$(echo "$col + 1" | bc)
            if test "$col" = "$colunas"
            then
                col=0
                lin=$(echo "$lin + 1" | bc)
            fi
        done
    done
}

tile_help()
{
    echo "use $(basename $0) opcoes
    --help          exibe esta mensagem de ajuda
    --horizontal    alinha as janelas em uma unica linha
    --vertical      alinha as janelas em uma unica coluna
    --matrix        alinha as janelas em uma matriz
    --test n        cria n janelas de teste"
}

tile_horizontal()
{
    n=$(get_janelas_do_desktop_atual | wc -l)
    tile 1 $n
}

tile_vertical()
{
    n=$(get_janelas_do_desktop_atual | wc -l)
    tile $n 1
}

tile_matrix()
{
    n=$(get_janelas_do_desktop_atual | wc -l)
    l=$(echo "sqrt($n)" | bc)
    c=$(echo "$n / $l" | bc)
    t=$(echo "$l * $c" | bc)
    if test "$t" != "$n"
    then
        l=$(echo "$l + 1" | bc)
    fi
    tile $l $c
}

tile_test()
{
    if test -z "$1"
    then
        echo "informe a quantidade de janelas de teste"
        exit 1
    fi
    if test "$1" -lt 1 -o "$1" -gt 16
    then
        echo "n deve ser entre 1 e 16"
        exit 1
    fi

    for i in $(seq $1)
    do
        xmessage $i &
    done
}

case "$1" in
    --help)
        tile_help
        ;;

    --horizontal)
        tile_horizontal
        ;;

    --vertical)
        tile_vertical
        ;;

    --matrix)
        tile_matrix
        ;;

    --test)
        tile_test "$2"
        ;;

    *)
        echo "use $0 {--help|--horizontal|--vertical|--matrix|--test}"
        exit 1
        ;;
esac
